/*
 * ConstraintCellRenderer.java
 *
 * Copyright (C) 2002-2013 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.executequery.gui.table;

import java.awt.Component;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import org.executequery.GUIUtilities;
import org.executequery.gui.browser.ColumnConstraint;

/**
 *
 * @author   Takis Diakoumis
 * @version  $Revision: 1094 $
 * @date     $Date: 2012-02-20 23:51:20 +1100 (Mon, 20 Feb 2012) $
 */
public class ConstraintCellRenderer extends JLabel
                                    implements TableCellRenderer {
    
    private static ImageIcon deleteImage;
    private static ImageIcon newImage;

    static {
        deleteImage = GUIUtilities.loadIcon("MarkDeleted16.png", true);
        newImage = GUIUtilities.loadIcon("MarkNew16.png", true);
    }

    /** Creates a new instance of ConstraintCellRenderer */
    public ConstraintCellRenderer() {}

    public Component getTableCellRendererComponent(JTable table,
                                Object value, boolean isSelected, boolean hasFocus,
                                int row, int column) {
        

        ColumnConstraint cc = (ColumnConstraint)value;
        if (cc.isMarkedDeleted()) {
            setIcon(deleteImage);
            setToolTipText("This value marked to be dropped");
        }
        else if (cc.isNewConstraint()) {
            setIcon(newImage);
            setToolTipText("This value marked new");            
        }
        else {
            setIcon(null);
        }

        setHorizontalAlignment(JLabel.CENTER);

        return this;
    }

}






